/*
 * getpw & getgr version 0.1
 * 
 * Copyright (c) 2003 Veli Mankinen (veli.mankinen@movial.fi)
 *
 * getpw & getgr are licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 * 
 **/

#include <stdlib.h>
#include <stdio.h>

#ifdef GR_MODE
#	include <grp.h>
#else
#	include <pwd.h>
#endif

#include <sys/types.h>
#include <ctype.h>
#include <errno.h>

int main(int argc, char *argv[])
{
#ifdef GR_MODE
	#define MODE "group"
	#define ID gid
	#define STRUCT grp
	#define GETNAM getgrnam
	#define GETID getgrgid
	
	struct group *grp;
	int gid;
	char *add;
	int i;
#else
	#define MODE "user"
	#define ID uid
	#define STRUCT pswd
	#define GETNAM getpwnam
	#define GETID getpwuid

	struct passwd *pswd;
	int uid;
#endif

	char *tmp;
	int nammode = 0;
	
	if (argc < 2) {
		fprintf(stderr, "Usage: %s [" MODE "name | " MODE "id]\n", argv[0]);
		return 1;
	}
	
	tmp = argv[1];
	while (*tmp != '\0') {
		if (isalpha(*tmp++)) {
			nammode = 1;
			break;
		}
	}
	
	if (nammode) STRUCT = GETNAM(argv[1]);
	else {
		ID = strtol(argv[1], (char **)NULL, 10);
		if (errno == ERANGE || errno == EINVAL) {
			fprintf(stderr, "Error converting argument to integer!\n");
			return 3;
		}
		STRUCT = GETID(ID);
	}
			
	if (STRUCT == NULL) {
		fprintf(stderr, "No such " MODE "!\n");
		return 2;
	}

#ifdef GR_MODE
	printf("gr_name: %s\n"
	       "gr_passwd: %s\n"
	       "gr_gid: %i\n"
	       "gr_mem: ",
	       grp->gr_name, grp->gr_passwd, grp->gr_gid);

	add = "";
	for (i=0; grp->gr_mem[i] != NULL; i++) {
		printf("%s'%s'", add, grp->gr_mem[i]);
		if (add[0] == '\0') add = ", ";	
	}

	printf("\n");
#else
	printf("pw_name: %s\n"
	       "pw_passwd: %s\n"
	       "pw_uid: %i\n"
	       "pw_gid: %i\n"
	       "pw_gecos: %s\n"
	       "pw_dir: %s\n"
	       "pw_shell: %s\n",
	       pswd->pw_name, pswd->pw_passwd, pswd->pw_uid,
	       pswd->pw_gid, pswd->pw_gecos, pswd->pw_dir,
	       pswd->pw_shell);
	
#endif
	
	return 0;
}

